package u7a4;

import reversi.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * HumanPlayer implementiert auf einfache Art das Interface für einen
 * Reversi-Spieler ({@link reversi.ReversiPlayer}). Anstatt einen Zug zu
 * berechnen, fordert <code>HumanPlayer</code> den Benutzer auf, einen Zug über
 * die Konsole einzugeben.
 *
 * @see reversi.ReversiPlayer
 * @author pascscha
 */
public class HumanPlayer implements ReversiPlayer {

	/**
	 * Die Farbe des Spielers. Entweder {@link reversi.GameBoard.RED} oder
	 * {@link reversi.GameBoard.GREEN}
	 */
	int color;

	/**
	 * Speichert die Farbe und den Timeout-Wert in Instanzvariablen ab. Diese
	 * Methode wird vor Beginn des Spiels von {@link Arena} aufgerufen.
	 *
	 * @see reversi.ReversiPlayer
	 */
	public void initialize(int myColor, long timeLimit) {
		color = myColor;
	}

	/**
	 * Macht einen Zug für den HumanPlayer, indem der Benutzer zur Eingabe eines
	 * Zuges aufgefordert wird. Diese Methode wird von {@link reversi.Arena}
	 * abwechselnd aufgerufen.
	 *
	 * @see reversi.ReversiPlayer
	 * @return Der Zug des HumanPlayers.
	 */
	public Coordinates nextMove(GameBoard gb) {

		// Check if the player has any legal moves
		if (gb.isMoveAvailable(color)) {

			System.out.print(String.format("Player %s: Where do you want to play?\n", Utils.toString(color)));

			// The Coordinates the user inputs
			Coordinates where = new Coordinates(-1, -1);

			// Read in coordinates until the user inputs a valid move
			while (!gb.checkMove(color, where)) {
				where = readCoordinatesFromKeyboard();

				// Print an error message if the provided position is invalid
				if (where != null && !gb.checkMove(color, where)) {
					System.out.println(
							String.format("Player %s can't play at Position %s. Please enter a valid position.",
									Utils.toString(color), where.toMoveString()));
				}
			}
			return where;
		} else {
			System.out.println(String.format("Player %s has no legal moves, passes.", Utils.toString(color)));
			return null;
		}
	} // end nextMove()


	/**
	 * Liest einen Zug vom Benutzer ein. Gültige Eingaben sind entweder ein
	 * Koordinatenpaar bestehend aus Zeile und Spalte (z.B. '6d') oder ein 'p',
	 * um zu passen falls kein Zug möglich ist. Methode wiederholt die
	 * Eingabeaufforderung so lange bis eine gültige Eingabe gemacht wurde.
	 *
	 * @return Gibt die eingelesenen Koordinaten zurück, bzw. <code>null</code>,
	 *         wenn die Benutzereingabe nicht korrekt war.
	 */
	private Coordinates readCoordinatesFromKeyboard() {
		// Scanner that can read values from Console
		BufferedReader scanner = new BufferedReader(new InputStreamReader(System.in));

		// The string which will hold the Input text
		String input;

		// Read the input and convert it to upper case, to make input case insensitive
		try {
			input = scanner.readLine().toUpperCase();
		} catch (IOException e) {
			System.err.println("IO Error");
			return null;
		}

		// If the input has invalid length, repeat
		if (input.length() != 2) {
			System.err.println("Invalid input. (Example Input: C6)");
			return null;
		}

		// Get the two input chars
		char first = input.charAt(0);
		char second = input.charAt(1);

		// Input has digit first and letter second (eg. 3F)
		if (first >= '1' && first <= '8' && second >= 'A' && second <= 'H') {
			return new Coordinates(first - '1' + 1, second - 'A' + 1);
		}
		// Input has letter first and digit second (eg. F3)
		else if (second >= '1' && second <= '8' && first >= 'A' && first <= 'H') {
			return new Coordinates(second - '1' + 1, first - 'A' + 1);
		}
		// Input is invalid
		else {
			System.err.println("Invalid input. (Example Input: C6)");
			return null;
		}
	} // end readCoordinatesFromKeyboard()

}
